from .upload_func import login, create, pull, update
from termcolor import colored
import json, os, tarfile, h5py, time, datetime

class UploadGenerator:
    def __init__(self, root, account_path=os.path.abspath("account.json")):
        self.account_path = account_path
        self.root = root
        self.ep_hdf5 = None
        self.task_id = ""
        self.ensure_login()
        self.upload_login()

    def ensure_login(self):
        if not os.path.exists(self.account_path):
            account_data = {}
            print(colored("empty username or password", "red"))
            account_data["username"] = input("username: ")
            account_data["password"] = input("password: ")
            with open(self.account_path, "w") as f:
                json.dump(account_data, f)

    def upload_login(self):
        with open(self.account_path, "r") as f:
            account_data = json.load(f)
        login(account_data["username"], account_data["password"])
        print(colored(f"hello {account_data['username']}!", "green"))
   
    def generate_file(self, task_root, ep_name, env_name, env_info):
        print(colored(f"Generating tar file for episode {ep_name}...", "yellow"))
        env_info = json.loads(env_info)
        env_meta = dict(
            **env_info,
            has_renderer=False,
            renderer="mjviewer",
            has_offscreen_renderer=True,
            use_camera_obs=False
        )
        env_meta["translucent_robot"] = False
        env_meta_all = dict(
            env_kwargs=env_meta,
            env_name=env_name
        )
        ep_hdf5_path = os.path.join(task_root, "episodes", ep_name, "ep_demo.hdf5")
        if self.ep_hdf5 is not None:
            self.ep_hdf5.close()
        self.ep_hdf5 = h5py.File(ep_hdf5_path, "r+")
        # check windows path problem: \\->/
        ep = list(self.ep_hdf5["data"].keys())[0]
        ep_meta = self.ep_hdf5["data/{}".format(ep)].attrs.get("ep_meta", None)
        if ep_meta is not None:
            ep_meta_json = json.loads(ep_meta)
            for obj_cfg in ep_meta_json["object_cfgs"]:
                if "\\" not in obj_cfg["info"]["mjcf_path"]: break # not windows path
                obj_cfg["info"]["mjcf_path"] = obj_cfg["info"]["mjcf_path"].replace("\\", "/")
            ep_meta = json.dumps(ep_meta_json)
            self.ep_hdf5["data/{}".format(ep)].attrs["ep_meta"] = ep_meta
            self.ep_hdf5.flush()
        # 获取episode目录路径
        ep_directory = os.path.join(task_root, "episodes", ep_name)
        if not os.path.exists(ep_directory):
            print(colored(f"Episode directory not found: {ep_directory}", "red"))
            return
        # 把环境信息写入ep hdf5
        self.ep_hdf5["data"].attrs["env_args"] = json.dumps(env_meta_all)
        self.ep_hdf5.close() # if windows, need to close before creating tar file
        # 创建tar文件路径
        tar_path = os.path.join(task_root, "episodes", f"{env_name}-{ep_name}.tar")
        try:
            with tarfile.open(tar_path, "w") as tar:
                tar.add(ep_directory, arcname=ep_name)
            print(colored(f"Saved tar file to {tar_path}", "green"))
        except Exception as e:
            print(colored(f"Error generating tar file: {str(e)}", "red"))

    def upload(self, task_root, ep_name, env_name, env_info, metadata):
        self.generate_file(task_root, ep_name, env_name, env_info)
        file_path = os.path.join(task_root, "episodes", f"{env_name}-{ep_name}.tar")
        video_path = ""
        create(f"{env_name}-{ep_name}", file_path, video_path, metadata, self.task_id)
        print(colored(f"Successfully uploaded", "green"))

    def pull_task(self, batch_name="", filters={}):
        task_info = pull(self.task_id, batch_name=batch_name, filters=filters)
        self.task_id = task_info["task_id"]
        assert self.task_id != "", colored("No task left", "red")
        task_info.pop("task_id")
        return task_info
    
    def update_task(self, mode="reset"):
        update(self.task_id, mode)

    def close(self):
        self.env.close()

if __name__ == "__main__":
    account_path = os.path.abspath("account.json")
    upload_generator = UploadGenerator(task_root="", env_name="", env_info="")
